package com.gree.mall.contest.config;

import com.gree.mall.contest.enums.base.BaseEnum;
import org.springframework.core.convert.converter.Converter;
import org.springframework.core.convert.converter.ConverterFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * @author qinrongjun
 * @description 枚举类型转换工厂
 * @date 2023/4/6 11:07 星期四
 */
public class BaseEnumConverterFactory implements ConverterFactory<String, BaseEnum> {
    /**
     * 缓存BaseEnum实现类
     */
    private static final Map<Class<?>, Converter<String, ? extends BaseEnum>> CACHE_MAP = new HashMap<>(32);

    @Override
    public <T extends BaseEnum> Converter<String, T> getConverter(Class<T> targetType) {
        return (Converter<String, T>) CACHE_MAP.computeIfAbsent(targetType, key -> new BaseEnumImpl<>(targetType));
    }

    private static class BaseEnumImpl<T extends BaseEnum> implements Converter<String, T> {

        private final Map<String, T> enumMap = new HashMap<>();

        private BaseEnumImpl(Class<T> clazz) {
            T[] enums = clazz.getEnumConstants();
            for (T t : enums) {
                enumMap.put(t.getKey(), t);
            }
        }

        @Override
        public T convert(String source) {
            return enumMap.get(source);
        }
    }
}
