package com.gree.mall.contest.config.aop;

import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.AdminUser;
import com.gree.mall.contest.plus.entity.OperationLog;
import com.gree.mall.contest.plus.service.AdminUserService;
import com.gree.mall.contest.plus.service.OperationLogService;
import com.gree.mall.contest.utils.CommonUtils;
import com.gree.mall.contest.utils.IpUtil;
import com.gree.mall.contest.utils.RedisUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 *  操作日志
 *  created by lijh
 *  2020-06-03
 */
@Aspect
@Slf4j
public class OperationLogAspect {

    @Autowired
    RedisUtil redisUtil;
    @Autowired
    OperationLogService operationLogService;
    @Autowired
    AdminUserService adminUserService;
    @Autowired
    CommonLogic commonLogic;

    @Pointcut("@annotation(io.swagger.v3.oas.annotations.Operation)")
    public void auth() {}

    @Before("auth()")
    public void doBefore(JoinPoint joinPoint){
        try{
            start(joinPoint);
        }catch(Exception e){
            log.error("操作日志记录失败,{}",e.getMessage());
        }
    }


    private void start(JoinPoint joinPoint){
        //不记录的模块
        List<String> modules = Arrays.asList("小程序API","公共API");

        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        String adminUserId = CommonUtils.getUserId(request);
        String nickName = CommonUtils.getRealName(request);
        Method sourceMethod = getSourceMethod(joinPoint);
        //只记录post请求的增删改操作
        PostMapping annotation = sourceMethod.getAnnotation(PostMapping.class);
        if (annotation != null) {
            Operation ApiOperation = sourceMethod.getAnnotation(Operation.class);
            //方法上的注解为具体操作
            String value = ApiOperation.summary();
            //类上的api注解为模块
            String moduleName = joinPoint.getTarget().getClass().getAnnotation(Tag.class).name();
            if(StringUtils.isBlank(moduleName) || modules.contains(moduleName)){
                return;
            }
            //登录接口获取不到token，写死
            if(value.equals("登录")){
                String userName = request.getParameter("userName");
                AdminUser adminUser = adminUserService.lambdaQuery().eq(AdminUser::getUserName, userName).one();
                nickName = adminUser.getNickName();
                adminUserId = adminUser.getAdminUserId();
            }
            //当前时间
            Date date = new Date();
            String dateStr = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date);

            //记录
            OperationLog operationLog = new OperationLog();
            operationLog.setContent(nickName + "于" + dateStr + value);
            operationLog.setModuleName(moduleName);
            operationLog.setAdminUserId(adminUserId);
            operationLog.setUserName(nickName);
            operationLog.setNickName(nickName);
            operationLog.setIp(IpUtil.getIpAddr(request));
            operationLog.setCreateTime(new Date());
            operationLogService.save(operationLog);
        }
    }

    private Method getSourceMethod(JoinPoint jp) {
        Method proxyMethod = ((MethodSignature) jp.getSignature()).getMethod();
        try {
            return jp.getTarget().getClass().getMethod(proxyMethod.getName(), proxyMethod.getParameterTypes());
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (SecurityException e) {
            e.printStackTrace();
        }
        return null;
    }

}
