package com.gree.mall.contest.controller.mini.common;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.bean.user.CurrentCompanyWechat;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.CarouselMapLogic;
import com.gree.mall.contest.logic.ExpressLogic;
import com.gree.mall.contest.logic.SMSLogic;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.logic.common.WechatLogic;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.AdminCompanyWechatOtherService;
import com.gree.mall.contest.plus.service.OrderDetailService;
import com.gree.mall.contest.plus.service.OrderInfoService;
import com.gree.mall.contest.plus.service.OrderTaxService;
import com.gree.mall.contest.utils.RedisUtil;
import com.gree.mall.contest.utils.VerifiUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;
import java.util.UUID;


@Slf4j
@RestController
@Tag(name = "公共API", description = "公共API")
@RequestMapping(value = "/mini/common", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class MiniCommonController {

    private final CommonLogic commonLogic;
    private final ExpressLogic expressLogic;
    private final OrderTaxService orderTaxService;
    private final RedisUtil redisUtil;
    private final CarouselMapLogic carouselMapLogic;
    private final WechatLogic wechatLogic;
    private final OrderInfoService orderInfoService;
    private final OrderDetailService orderDetailService;
    private final AdminCompanyWechatOtherService adminCompanyWechatOtherService;
    private final SMSLogic smsLogic;

    @PostMapping("/upload")
    @Operation(summary = "文件上传")
    public ResponseHelper<CommonFile> upload(
            @Parameter(description = "附件", required = true) @RequestParam(required = true) MultipartFile file
    ) throws Exception {
        CommonFile commonFile = commonLogic.uploadFile(file);
        return ResponseHelper.success(commonFile);
    }

    @GetMapping("/express/company")
    @Operation(summary = "所有物流公司")
    public ResponseHelper<List<ExpressCompany>> companyList(
    ) throws Exception {
        List<ExpressCompany> expressCompanies = expressLogic.expressCompanyList();
        return ResponseHelper.success(expressCompanies);
    }

    @GetMapping("/express")
    @Operation(summary = "查看物流")
    public ResponseHelper<List<ExpressInfo>> express(
            @Parameter(description = "附件", required = true) @RequestParam(required = true) String logisticsNo,
            @Parameter(description = "物流公司code", required = true) @RequestParam(required = true) String companyCode
    ) throws Exception {
        List<ExpressInfo> expressInfos = expressLogic.queryExpress(logisticsNo, companyCode);
        return ResponseHelper.success(expressInfos);
    }


    @GetMapping("/street")
    @Operation(summary = "街道")
    public ResponseHelper<List<Map>> street(
            @Parameter(description = "省", required = true) @RequestParam(required = true) String province,
            @Parameter(description = "市", required = true) @RequestParam(required = true) String city,
            @Parameter(description = "区", required = true) @RequestParam(required = true) String area
    ) throws RemoteServiceException {
        List<Map> street = commonLogic.street(province, city, area);
        return ResponseHelper.success(street);
    }

    @GetMapping("/list/page")
    @Operation(summary = "轮播图列表")
    public ResponseHelper<Page<CarouselMap>> page(
            HttpServletRequest request,
            @Parameter(description = "状态(true：启用 false:禁用", required = false) @RequestParam(required = false) Boolean state,
            @Parameter(description = "页号", required = true) @RequestParam(required = true) Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam(required = true) Integer pageSize
    ) throws RemoteServiceException {
        IPage<CarouselMap> page = carouselMapLogic.page(request, state, pageNum, pageSize);
        return ResponseHelper.success(page);
    }

    @GetMapping("/wechat/detail")
    @Operation(summary = "微信配置")
    public ResponseHelper<CurrentCompanyWechat> detail(
            HttpServletRequest request
    ) throws RemoteServiceException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        //设置其他配置
        AdminCompanyWechatOther one = adminCompanyWechatOtherService.lambdaQuery()
                .eq(AdminCompanyWechatOther::getCompanyWechatId, currentCompanyWechat.getCompanyWechatId())
                .one();
        currentCompanyWechat.setAdminCompanyWechatOther(one);
        return ResponseHelper.success(currentCompanyWechat);
    }


    @GetMapping("/scene")
    @Operation(summary = "获取微信scene参数")
    public ResponseHelper<String> page(
            @Parameter(description = "微信scene", required = true) @RequestParam(required = true) String scene
    ) throws RemoteServiceException {
        String sceneValue = commonLogic.getSceneValue(scene);
        return ResponseHelper.success(sceneValue);
    }

    @GetMapping("/getVerifi")
    @Operation(summary = "获取拖拽式验证码")
    public ResponseHelper<Map<String, Object>> getVerifi() {
        Map<String, Object> map = VerifiUtils.getVerifi();
        Integer xWidth = (Integer) map.get("xWidth");
        String key = UUID.randomUUID().toString();
        redisUtil.set(Constant.RedisPrefix.VERIFICATION + ":" + key, xWidth, 2 * 60);
        map.remove("xWidth");
        map.put("key", key);
        return ResponseHelper.success(map);
    }

    @ApiNotAuth
    @PostMapping("/sms/send")
    @Operation(summary = "发送验证码")
    public ResponseHelper sendSms(
            @Parameter(description = "手机号码", required = true) @RequestParam String phone,
            @Parameter(description = "滑动验证key") @RequestParam(required = false) String key,
            @Parameter(description = "滑动验证数值") @RequestParam(required = false) String vrifyCode,
            @Parameter(description = "发送类型 LOGIN=手机号登录 RESET=找回密码") @RequestParam(required = false, defaultValue = "LOGIN") String sendType
    ) throws Exception {
        smsLogic.sendSms(phone, key, vrifyCode, sendType);
        return ResponseHelper.success();
    }

    @GetMapping("/config/get")
    @Operation(summary = "获取小程序配置")
    public ResponseHelper<AdminCompanyWechat> getConfig(HttpServletRequest request) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        //剔除不返回的内容
        currentCompanyWechat.setPubAppId(null);
        currentCompanyWechat.setSubAppId(null);
        currentCompanyWechat.setMchKey(null);
        currentCompanyWechat.setMchId(null);
        return ResponseHelper.success(currentCompanyWechat);
    }


    @ApiNotAuth
    @GetMapping("/getfile")
    public ResponseHelper<String> getfile(@RequestParam(required = true) String id) {
        OrderTax orderTax = orderTaxService.getById(id);
        String taxLink = orderTax.getTaxLink();
        return ResponseHelper.success(taxLink);
    }

    @ApiNotAuth
    @GetMapping("/oss/config")
    public ResponseHelper getOSSConfig() throws UnsupportedEncodingException {
        Map<String, String> ossConfig = commonLogic.getOSSConfig();
        return ResponseHelper.success(ossConfig);
    }

    @ApiNotAuth
    @GetMapping("/img/get")
    @Operation(summary = "获取图片")
    public void getImg(
            @Parameter(description = "附件key", required = true) @RequestParam String key,
            HttpServletResponse response
    ) throws IOException, RemoteServiceException {
        String file = commonLogic.getFile(key);
        response.sendRedirect(file);
    }

    @ApiNotAuth
    @GetMapping("/region/list")
    @Operation(summary = "地区列表")
    public ResponseHelper<List<Region>> regionList(
            @Parameter(description = "父id(第一级传0)", required = true) @RequestParam String pid
    ) throws RemoteServiceException {
        List<Region> list = commonLogic.queryRegionList(Lists.newArrayList(pid));
        return ResponseHelper.success(list);
    }

}
