package com.gree.mall.contest.controller.mini.common;

import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.logic.common.WechatLogic;
import com.gree.mall.contest.logic.pay.PayLogic;
import com.gree.mall.contest.utils.WeChatUtils;
import io.swagger.v3.oas.annotations.Operation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@RequestMapping(value = "/pay", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class PayController {

    private final PayLogic payLogic;
    private final WechatLogic wechatLogic;
    private final RedisLockRegistry redisLockRegistry;
    static ExecutorService threadPool = Executors.newFixedThreadPool(4);


    @ApiNotAuth
    @PostMapping("/payCall")
    @Operation(summary = "支付回调")
    public String payCall(@RequestBody String body) throws Exception {
        Map<String, String> params = wechatLogic.doXMLParse(body);
        String id = params.get("out_trade_no");//订单号
        String transactionId = params.get("transaction_id");//微信支付单号
        BigDecimal totalFee = new BigDecimal(params.get("total_fee")).divide(BigDecimal.valueOf(100));//
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + id);

        threadPool.submit(() -> {
            try {
                log.info("==========支付回调开始：{}", body);
                if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
                    log.error("【支付回调】，请勿重复请求，orderId:{}", id);
                } else {
                    payLogic.payCall(id, transactionId, totalFee);
                }
            } catch (Exception e) {
                log.error("处理支付回调失败", e);
            } finally {
                if (obtain != null) {
                    obtain.unlock();
                }
            }
        });

        SortedMap<Object, Object> returnData = new TreeMap<>();
        returnData.put("return_code", "SUCCESS");
        returnData.put("return_msg", "OK");
        return WeChatUtils.getRequestXml(returnData);
    }


    @ApiNotAuth
    @PostMapping("/refundCall")
    @Operation(summary = "退款回调")
    public String refundCall(@RequestBody String body) throws Exception {
        SortedMap<Object, Object> returnData = new TreeMap<>();
        returnData.put("return_code", "SUCCESS");
        returnData.put("return_msg", "OK");
        return WeChatUtils.getRequestXml(returnData);
    }


}
