package com.gree.mall.contest.controller.mini.order;

import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.order.OrderTaxLogic;
import com.gree.mall.contest.plus.entity.OrderTax;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;

@Slf4j
@RestController
@Tag(name = "小程序用户发票", description = "小程序用户发票")
@RequestMapping(value = "/mini/user/order/tax", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class MiniOrderTaxController {

    private final OrderTaxLogic orderTaxLogic;

    @PostMapping("/save")
    @Operation(summary = "添加发票")
    public ResponseHelper<OrderTax> saveTax(
            @Parameter(description = "object", required = true) @RequestBody OrderTax orderTaxBean,
            HttpServletRequest request
    ) throws Exception {
        if (orderTaxBean.getTaxType()) {
            if (StringUtils.isEmpty(orderTaxBean.getReceiverPhone())) {
                throw new RemoteServiceException("请添加收件人电话");
            }
            if (StringUtils.isEmpty(orderTaxBean.getReceiverAddress())) {
                throw new RemoteServiceException("请添加收件人地址");
            }
            if (StringUtils.isEmpty(orderTaxBean.getReceiverName())) {
                throw new RemoteServiceException("请添加收件人姓名");
            }
            if(StringUtils.isEmpty(orderTaxBean.getRegisterAddress())) {
                throw new RemoteServiceException("请添加注册地址");
            }
            if(StringUtils.isEmpty(orderTaxBean.getRegisterPhone())){
                throw new RemoteServiceException("请添加注册电话");
            }
            if(StringUtils.isEmpty(orderTaxBean.getBank())){
                throw new RemoteServiceException("请添加开户银行");
            }
            if(StringUtils.isEmpty(orderTaxBean.getAccount())){
                throw new RemoteServiceException("请添加开户帐号");
            }
        }
        //判断纳税识别号长度
        if(orderTaxBean.getTaxNo() != null && (orderTaxBean.getTaxNo().length() < 15 || orderTaxBean.getTaxNo().length() > 20)){
            throw new RemoteServiceException("请填写15至20位长度的纳税识别号");
        }
        OrderTax orderTax = orderTaxLogic.saveTax(orderTaxBean);
        return ResponseHelper.success(orderTax);
    }

    @PostMapping("/update")
    @Operation(summary = "修改发票")
    public ResponseHelper<OrderTax> updateTax(
            @RequestBody OrderTax orderTax
    ) throws Exception {
        orderTaxLogic.miniUpdateTax(orderTax);
        return ResponseHelper.success();
    }

    @PostMapping("/del")
    @Operation(summary = "删除发票")
    public ResponseHelper<String> delTax(
            @Parameter(description = "发票id", required = true) @RequestParam(required = true) String orderTaxId
    ) throws Exception {
        return ResponseHelper.success(orderTaxLogic.delTax(orderTaxId));
    }

    @PostMapping("/send")
    @Operation(summary = "发送发票到email")
    public ResponseHelper<String> sendTax(
            @Parameter(description = "发票id", required = true) @RequestParam(required = true) String orderTaxId
    ) throws Exception {
        String addressId = orderTaxLogic.sendTax(orderTaxId);
        return ResponseHelper.success(addressId);
    }


    @ApiNotAuth
    @PostMapping("/yonge")
    @Operation(summary = "拆分邮件发送")
    public ResponseHelper yonge(
            @Parameter(description = "excel文件", required = true) @RequestParam(required = true) MultipartFile file
    ) throws Exception {
        orderTaxLogic.yonge(file);
        return ResponseHelper.success();
    }


    @GetMapping("/list")
    @Operation(summary = "发票列表")
    public ResponseHelper<Page<OrderTax>> pageTax(
            @Parameter(description = "用户id", required = true) @RequestParam(required = true) String userId,
            @Parameter(description = "页号", required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam Integer pageSize
    ) throws Exception {

        IPage<OrderTax> page = orderTaxLogic.miniPage(userId, pageNum, pageSize);
        return ResponseHelper.success(page);
    }

    @GetMapping("/detail")
    @Operation(summary = "发票详情")
    public ResponseHelper<OrderTax> detail(
            @Parameter(description = "发票id", required = true) @RequestParam(required = true) String orderTaxId
    ) throws Exception {
        OrderTax orderTax = orderTaxLogic.detail(orderTaxId);
        return ResponseHelper.success(orderTax);
    }


    @GetMapping("/title/list")
    @Operation(summary = "发票抬头列表")
    public ResponseHelper<String> title(
            @Parameter(description = "公司名称", required = true) @RequestParam(required = true) String companyName
    ) throws Exception {
        Object object = orderTaxLogic.title(companyName);
        return ResponseHelper.success(object);
    }


}
