package com.gree.mall.contest.controller.mini.order;

import com.gree.mall.contest.bean.order.ShoppingCartBean;
import com.gree.mall.contest.bean.order.ShoppingCartDetail;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.order.ShoppingCartLogic;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@Slf4j
@RestController
@Tag(name = "APP师傅端-购物车", description = "APP师傅端-购物车")
@RequestMapping(value = "/mini/shpping/cart", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class MiniShoppingCartController {

    private final ShoppingCartLogic shoppingCartLogic;

    @GetMapping("/notice")
    @Operation(summary = "消息通知")
    public ResponseHelper<String> notice(HttpServletRequest request) {
        return ResponseHelper.success(shoppingCartLogic.miniNotice(request));
    }


    @GetMapping("/list")
    @Operation(summary = "购物车列表")
    public ResponseHelper<ShoppingCartDetail> list(
            @Parameter(description = "用户id", required = true) @RequestParam String userId
    ) throws InterruptedException {
        ShoppingCartDetail shoppingCartLists = shoppingCartLogic.queryShoppingCart(userId);
        return ResponseHelper.success(shoppingCartLists);
    }

    @PostMapping("/add/one")
    @Operation(summary = "商品详情添加商品购物车")
    public ResponseHelper<ShoppingCartDetail> addOne(HttpServletRequest request, @RequestBody ShoppingCartBean shoppingCartBean) throws RemoteServiceException, InterruptedException {
        ShoppingCartDetail shoppingCartDetail = shoppingCartLogic.addOne(request, shoppingCartBean);
        return ResponseHelper.success(shoppingCartDetail);
    }


    @PostMapping("/add")
    @Operation(summary = "添加到购物车")
    public ResponseHelper<ShoppingCartDetail> addShoppingCart(HttpServletRequest request, @RequestBody ShoppingCartBean shoppingCartBean) throws RemoteServiceException, InterruptedException {
        ShoppingCartDetail shoppingCartDetail = shoppingCartLogic.addShoppingCart(request, shoppingCartBean);
        return ResponseHelper.success(shoppingCartDetail);
    }

    @PostMapping("/clear")
    @Operation(summary = "清空购物车")
    public ResponseHelper clearShoppingCart(
            @Parameter(description = "用户id", required = true) @RequestParam String userId
    ) {
        shoppingCartLogic.clearShoppingCart(userId);
        return ResponseHelper.success();
    }

    @PostMapping("/remove")
    @Operation(summary = "删除购物车商品")
    public ResponseHelper<ShoppingCartDetail> clearShoppingCart(
            @Parameter(description = "用户id", required = true) @RequestParam String userId,
            @Parameter(description = "购物车ids", required = true) @RequestParam List<String> shoppingCartIds
    ) throws InterruptedException {
        ShoppingCartDetail remove = shoppingCartLogic.remove(userId, shoppingCartIds);
        return ResponseHelper.success(remove);
    }

}
