package com.gree.mall.contest.controller.pc;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.CarouselMapLogic;
import com.gree.mall.contest.plus.entity.CarouselMap;
import com.gree.mall.contest.plus.entity.Goods;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Slf4j
@RestController
@Tag(name = "轮播图管理", description = "轮播图管理")
@RequestMapping(value = "/pc/carouselMap", produces = "application/json; charset=utf-8")
public class CarouselMapController {

    @Resource
    CarouselMapLogic carouselMapLogic;

    @GetMapping("/list/page")
    @Operation(summary = "轮播图列表")
    public ResponseHelper<Page<CarouselMap>> page(
            HttpServletRequest request,
            @Parameter(description = "状态(true：启用 false:禁用") @RequestParam(required = false) Boolean state,
            @Parameter(description = "页号",required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小",required = true) @RequestParam Integer pageSize
    ) throws RemoteServiceException {
        IPage<CarouselMap> page = carouselMapLogic.page(request, state,pageNum, pageSize);
        return ResponseHelper.success(page);
    }

    @GetMapping("/detail")
    @Operation(summary = "轮播详情")
    public ResponseHelper<CarouselMap> detail(
            @Parameter(description = "id",required = true) @RequestParam() String id
    ) throws RemoteServiceException {
        CarouselMap detail = carouselMapLogic.detail(id);
        return ResponseHelper.success(detail);
    }


    @PostMapping("/add")
    @Operation(summary = "新增轮播图")
    public ResponseHelper add(
            HttpServletRequest request,
            @Parameter(description = "object",required = true) @RequestBody CarouselMap carouselMapBean) {
        carouselMapLogic.add(request, carouselMapBean);
        return ResponseHelper.success();
    }

    @PostMapping("/update")
    @Operation(summary = "修改轮播图")
    public ResponseHelper update(
            @Parameter(description = "object",required = true) @RequestBody CarouselMap carouselMapBean) {
        carouselMapLogic.update(carouselMapBean);
        return ResponseHelper.success();
    }

    @PostMapping("/delete")
    @Operation(summary = "删除轮播图")
    public ResponseHelper delete(
            @Parameter(description = "轮播图id",required = true) @RequestParam String carouselMapId
    ) {
        carouselMapLogic.delete(carouselMapId);
        return ResponseHelper.success();
    }

    @PostMapping("/batch/delete")
    @Operation(summary = "删除轮播图")
    public ResponseHelper delete(
            @Parameter(description = "轮播图id",required = true) @RequestBody List<String> ids
    ) {
        carouselMapLogic.delete(ids);
        return ResponseHelper.success();
    }


    @PostMapping("/state/update")
    @Operation(summary = "禁用/正常")
    public ResponseHelper change(
            @Parameter(description = "轮播图id",required = true) @RequestParam String carouselMapId,
            @Parameter(description = "状态:true上架 false下架",required = true) @RequestParam Boolean state
    ) {
        carouselMapLogic.updateState(carouselMapId, state);
        return ResponseHelper.success();
    }

    @GetMapping("/list/goods")
    @Operation(summary = "商品列表")
    public ResponseHelper<Page<Goods>> page(
            HttpServletRequest request,
            @Parameter(description = "商品id") @RequestParam(required = false) String goodsId,
            @Parameter(description = "商品名称") @RequestParam(required = false) String goodsName,
            @Parameter(description = "页号",required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小",required = true) @RequestParam Integer pageSize
    ) throws RemoteServiceException {
        IPage<Goods> page = carouselMapLogic.getGoods(request, goodsId, goodsName,pageNum, pageSize);
        return ResponseHelper.success(page);
    }
}
