package com.gree.mall.contest.controller.pc.goods;

import cn.hutool.core.lang.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.annotation.ZfireList;
import com.gree.mall.contest.bean.goods.*;
import com.gree.mall.contest.bean.zfire.ZfireParamBean;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.goods.GoodsCategoryLogic;
import com.gree.mall.contest.logic.goods.GoodsLogic;
import com.gree.mall.contest.plus.entity.GoodsCategory;
import com.gree.mall.contest.plus.entity.GoodsShare;
import com.gree.mall.contest.utils.zfire.FieldUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Slf4j
@RestController
@Tag(name = "分享图片管理API", description = "分享图片管理API")
@RequestMapping(value = "/pc/goods/share", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class GoodsShareController {

    private final GoodsLogic goodsLogic;

    @ZfireList
    @PostMapping("/list")
    @Operation(summary = "分享图片列表V2")
    public ResponseHelper<IPage<GoodsShareVO>> list(
            @RequestBody ZfireParamBean zfireParamBean
    ) throws RemoteServiceException {
        ZfireParamBean zfireParam = FieldUtils.supplyParam(zfireParamBean);
        IPage<GoodsShareVO> adminDeptVOIPage = goodsLogic.goodsSharePage(new Page(zfireParam.getPageNum(), zfireParam.getPageSize()), zfireParam);
        return ResponseHelper.success(adminDeptVOIPage, new TypeReference<GoodsShareVO>() {});
    }

    @PostMapping("/list/export")
    @Operation(summary = "分享图片列表V2导出")
    public void listExport(
            @RequestBody ZfireParamBean zfireParamBean,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        ZfireParamBean zfireParam = FieldUtils.supplyParam(zfireParamBean);
        IPage<GoodsShareVO> adminDeptVOIPage = goodsLogic.goodsSharePage(new Page(zfireParam.getPageNum(), zfireParam.getPageSize()), zfireParam);
        //3.导出
        FieldUtils.exportData(adminDeptVOIPage.getRecords(), zfireParam.getExportFields(), request, response);
    }
    @PostMapping("/add")
    @Operation(summary = "新增分享图片")
    public ResponseHelper add(
            @Parameter(description = "object",required = true) @RequestBody GoodsShareDTO goodsShareDTO, HttpServletRequest request) {
        goodsLogic.addShare(goodsShareDTO,request);
        return ResponseHelper.success();
    }

    @PostMapping("/update")
    @Operation(summary = "修改分享图片")
    public ResponseHelper update(
            @Parameter(description = "object",required = true) @RequestBody GoodsShareDTO goodsShareDTO) {
        goodsLogic.updateShare(goodsShareDTO);
        return ResponseHelper.success();
    }

    @PostMapping("/delete")
    @Operation(summary = "删除分享图片")
    public ResponseHelper delete(
            @Parameter(description = "分享id",required = true) @RequestParam String shareId
    ) {
        goodsLogic.deleteShare(shareId);
        return ResponseHelper.success();
    }

    @GetMapping("/detail")
    @Operation(summary = "详情")
    public ResponseHelper<GoodsShareDetail> detail(
            @Parameter(description = "分享id",required = true) @RequestParam String shareId
    ){
        GoodsShareDetail detail = goodsLogic.detailShare(shareId);
        return ResponseHelper.success(detail);
    }
}
