package com.gree.mall.contest.controller.pc.notice;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.notice.NoticeLogic;
import com.mysql.cj.protocol.x.Notice;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@Slf4j
@RestController
@Tag(name = "消息通知接口", description = "消息通知接口")
@RequestMapping(value = "/pc/notice/", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class NoticeController {
    
    private final NoticeLogic noticeLogic;

    @GetMapping("/list")
    @Operation(description = "获取消息列表")
    public ResponseHelper<IPage<Notice>> list(HttpServletRequest request,
                                              @Parameter(description = "已读标记true:已读,false:未读,不传获取全部消息", required = false) @RequestParam(required = false) Boolean readFlag,
                                              @Parameter(description = "页号", required = true) @RequestParam Integer pageNum,
                                              @Parameter(description = "页大小", required = true) @RequestParam Integer pageSize
    ) throws Exception {
        return ResponseHelper.success(noticeLogic.page(request,readFlag, pageNum, pageSize));
    }


    @GetMapping("/list/count")
    @Operation(description = "未读消息数量")
    public ResponseHelper<Integer> unreadCount(HttpServletRequest request) throws Exception {
        return ResponseHelper.success(noticeLogic.unreadCount(request));
    }


    @PostMapping("/mark/read")
    @Operation(description = "标记已读")
    public ResponseHelper markRead(HttpServletRequest request,@Parameter(description = "通知消息id数组", required = true) @RequestBody String[] noticeIds) throws Exception {
        noticeLogic.markRead(noticeIds);
        return ResponseHelper.success();
    }


    @PostMapping("/mark/read/all")
    @Operation(description = "全部标记已读")
    public ResponseHelper markReadAll(HttpServletRequest request) throws Exception {
        noticeLogic.markReadAll(request);
        return ResponseHelper.success();
    }


}
