package com.gree.mall.contest.controller.pc.order;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.gree.mall.contest.bean.order.BatchRemarkBean;
import com.gree.mall.contest.bean.order.OrderDetailBean;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.order.OrderLogic;
import com.gree.mall.contest.plus.entity.OrderInfo;
import com.gree.mall.contest.plus.entity.OrderRefundCliaims;
import com.gree.mall.contest.utils.CommonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@Tag(name = "订单管理", description = "订单管理")
@RequestMapping(value = "/pc/order", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class OrderController {

    private final OrderLogic orderLogic;
    private final RedisLockRegistry redisLockRegistry;

    @GetMapping("/list")
    @Operation(summary ="列表")
    public ResponseHelper<Page<OrderDetailBean>> list(
            @Parameter(description = "用户id", required = false) @RequestParam(required = false) String userId,
            @Parameter(description = "订单号", required = false) @RequestParam(required = false) String orderId,
            @Parameter(description = "产品名称", required = false) @RequestParam(required = false) String productName,
            @Parameter(description = "用户名", required = false) @RequestParam(required = false) String userName,
            @Parameter(description = "手机号", required = false) @RequestParam(required = false) String phone,
            @Parameter(description = "兑换码", required = false) @RequestParam(required = false) String exchangeCode,
            @Parameter(description = "订单状态 NOPAY:待付款 DFH:待发货 YFH:已发货 OVER:已完成 CLOSE:已关闭", required = false) @RequestParam(required = false) String orderStatus,
            @Parameter(description = "开始创建时间", required = false) @RequestParam(required = false) String startCreateTime,
            @Parameter(description = "结束创建时间", required = false) @RequestParam(required = false) String endCreateTime,
            @Parameter(description = "开始支付时间", required = false) @RequestParam(required = false) String startPayTime,
            @Parameter(description = "结束支付时间", required = false) @RequestParam(required = false) String endPayTime,
            @Parameter(description = "网点id", required = false) @RequestParam(required = false) String websitId,
            @Parameter(description = "页号", required = true) @RequestParam(required = true) Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam(required = true) Integer pageSize,
            @Parameter(description = "是否是差评订单0:未评价订单 1:差评订单 2:好评订单 3：已评价订单  不传都有", required = false) @RequestParam(required = false) String isNegative,
            @Parameter(description = "派工异常 true为派工异常订单", required = false) @RequestParam(required = false) String abnormalDispatch,
            @Parameter(description = "信息编号", required = false) @RequestParam(required = false) String workerOrderNo,
            @Parameter(description = "套购商品id", required = false) @RequestParam(required = false) String goodsId,
             HttpServletRequest request
    ) throws RemoteServiceException {
        IPage<OrderDetailBean> list = orderLogic
                .list(request,userId,orderId, productName, userName, phone,exchangeCode, orderStatus, startCreateTime,
                        endCreateTime, startPayTime, endPayTime,websitId,pageNum, pageSize,isNegative,abnormalDispatch,workerOrderNo,goodsId);
        return ResponseHelper.success(list);
    }

    @GetMapping("/detail")
    @Operation(summary ="订单详情")
    public ResponseHelper<OrderDetailBean> detail(
            @Parameter(description = "订单号", required = true) @RequestParam(required = true) String orderId
    ) {
        OrderDetailBean detail = orderLogic.detail(orderId);
        return ResponseHelper.success(detail);
    }


    @PostMapping("/comment/show")
    @Operation(summary ="显示评价")
    public ResponseHelper showComment(
            @Parameter(description = "订单号", required = true) @RequestParam(required = true) String orderId,
            @Parameter(description = "true=显示 false=不显示", required = true) @RequestParam(required = true) Boolean isShow
    ) {
        orderLogic.updateShowComment(orderId,isShow);
        return ResponseHelper.success();
    }

    @PostMapping("/remark")
    @Operation(summary ="备注")
    public ResponseHelper remark(
            @Parameter(description = "订单号", required = true) @RequestParam(required = true) String orderId,
            @Parameter(description = "备注", required = true) @RequestParam(required = true) String remark
    ) {
        orderLogic.remark(orderId, remark);
        return ResponseHelper.success();
    }

    @PostMapping("/express/update")
    @Operation(summary ="修改物流单号")
    public ResponseHelper updateExpress(
            @Parameter(description = "订单号", required = true) @RequestParam(required = true) String orderId,
            @Parameter(description = "物流单号", required = true) @RequestParam(required = true) String logisticsNo,
            @Parameter(description = "物流公司code", required = true) @RequestParam(required = true) String companyCode
    ) throws RemoteServiceException {
        orderLogic.updateExpress(orderId, logisticsNo,companyCode);
        return ResponseHelper.success();
    }

    @GetMapping("/expressInfo/select")
    @Operation(summary ="查看物流")
    public ResponseHelper queryExpressInfo(
            @Parameter(description = "物流单号", required = true) @RequestParam(required = true) String logisticsNo
    ) throws RemoteServiceException {
        orderLogic.queryExpressInfo(logisticsNo);
        return ResponseHelper.success();
    }


    @PostMapping("/cancel")
    @Operation(summary ="取消订单")
    public ResponseHelper remark(
            @Parameter(description = "订单号", required = true) @RequestParam(required = true) String orderId
    ) throws RemoteServiceException, WxPayException {
        orderLogic.cancel(orderId);
        return ResponseHelper.success();
    }

    @PostMapping("/ack")
    @Operation(summary ="确认收款")
    public ResponseHelper ack(
            @Parameter(description = "订单号", required = true) @RequestParam(required = true) String orderId
    ) {
        orderLogic.ackPay(orderId);
        return ResponseHelper.success();
    }



    @PostMapping("/detail/refund")
    @Operation(summary ="订单详情主动退款")
    public ResponseHelper<BigDecimal> notice(
            @Parameter(description = "订单详情id",required = true) @RequestParam String orderDetailId,
            @Parameter(description = "数量",required = true) @RequestParam Integer num,
            @Parameter(description = "退款密码") @RequestParam(required = false) String refundPassword
    ) throws Exception {
        BigDecimal refund = orderLogic.refund(orderDetailId, num, refundPassword);
        return ResponseHelper.success(refund);
    }

    @GetMapping("/cliaims/list")
    @Operation(summary ="理赔列表")
    public ResponseHelper<List<OrderRefundCliaims>> cliaimslist(
            @Parameter(description = "订单id",required = true) @RequestParam String orderId
    ) throws RemoteServiceException, WxPayException {
        List<OrderRefundCliaims> orderRefundCliaims = orderLogic.cliaimsList(orderId);
        return ResponseHelper.success(orderRefundCliaims);
    }

    @PostMapping("/cliaims/refund")
    @Operation(summary ="订单详情理赔")
    public ResponseHelper refundcliaims(
            @Parameter(description = "订单详情id",required = true) @RequestParam String orderDetailId,
            @Parameter(description = "数量",required = true) @RequestParam Integer num,
            @Parameter(description = "退款金额",required = true) @RequestParam BigDecimal refundAmount,
            @Parameter(description = "登录人帐号id",required = true) @RequestParam String adminUserId,
            @Parameter(description = "理赔备注",required = true) @RequestParam String remark,
            @Parameter(description = "退款密码") @RequestParam(required = false) String refundPassword
    ) throws Exception {
        orderLogic.refundCliaims(orderDetailId, num, refundAmount, adminUserId, remark, refundPassword);
        return ResponseHelper.success();
    }


    @PostMapping("/price/update")
    @Operation(summary ="修改价格")
    public ResponseHelper upatePrice(
            @Parameter(description = "订单号", required = true) @RequestParam(required = true) String orderId,
            @Parameter(description = "价格", required = true) @RequestParam(required = true) BigDecimal price
    ) throws RemoteServiceException {
        orderLogic.updatePrice(orderId, price);
        return ResponseHelper.success();
    }

    @PostMapping("/rece/info/update")
    @Operation(summary ="收货信息修改")
    public ResponseHelper updateReceInfo(
            @Parameter(description = "订单", required = true) @RequestBody OrderInfo orderInfo
    ) throws RemoteServiceException {
        orderLogic.updateReceInfo(orderInfo);
        return ResponseHelper.success();
    }

    @PostMapping("/goods/deliver")
    @Operation(summary ="发货")
    public ResponseHelper deliverGoods(
            @Parameter(description = "订单号", required = true) @RequestParam(required = true) String orderId,
            @Parameter(description = "快递单号", required = true) @RequestParam(required = true) String logisticsNo,
            @Parameter(description = "货仓id", required = true) @RequestParam(required = true) String storageId,
            @Parameter(description = "快递公司编号", required = true) @RequestParam(required = true) String companyCode,
            @Parameter(description = "是否需要派单 true=是 false=否", required = false) @RequestParam(required = false) Boolean workOrder,
            @Parameter(description = "是否退换单 true=是 false=否", required = false) @RequestParam(required = false) Boolean exchange
    ) throws Exception {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + orderId);
        if(!obtain.tryLock(10,TimeUnit.SECONDS)){
            return ResponseHelper.error("系统繁忙，请稍后再试");
        }
        try {
            orderLogic.deliverGoods(orderId, logisticsNo, companyCode, storageId, workOrder, exchange);
        }finally {
            obtain.unlock();
        }
        return ResponseHelper.success();
    }

    @PostMapping("/goods/batch/deliver")
    @Operation(summary ="批量发货")
    public ResponseHelper deliverBatchGoods(
            HttpServletRequest request,
            @Parameter(required = true, description = "excel文件") @RequestParam(required = true) MultipartFile file
    ) throws Exception {
        orderLogic.deliverBatchGoods(request, file);
        return ResponseHelper.success();
    }


    @GetMapping("/goods/deliver/excel")
    @Operation(summary ="批量发货excel示例表格下载")
    public void excelDownload(
            HttpServletResponse response
    ) throws RemoteServiceException, IOException {
        CommonUtils.downloadFile("/static/批量发货模板.xlsx", response);
    }


    @PostMapping("/batch/remark")
    @Operation(summary ="卖家批量设置备注")
    public ResponseHelper batchRemark(
            @Parameter(description = "备注信息", required = true) @RequestBody BatchRemarkBean batchRemarkBean
    ) throws RemoteServiceException {
        orderLogic.batchRemark(batchRemarkBean);
        return ResponseHelper.success();
    }


    @GetMapping("/export")
    @Operation(summary ="导出excel")
    public void downloadFile2 (HttpServletRequest request, HttpServletResponse response,
                               @Parameter(description = "用户id", required = false) @RequestParam(required = false) String userId,
                               @Parameter(description = "订单号", required = false) @RequestParam(required = false) String orderId,
                               @Parameter(description = "产品名称", required = false) @RequestParam(required = false) String productName,
                               @Parameter(description = "用户名", required = false) @RequestParam(required = false) String userName,
                               @Parameter(description = "手机号", required = false) @RequestParam(required = false) String phone,
                               @Parameter(description = "兑换码", required = false) @RequestParam(required = false) String exchangeCode,
                               @Parameter(description = "订单状态 NOPAY:待付款 DFH:待发货 YFH:已发货 OVER:已完成 CLOSE:已关闭", required = false)
                                   @RequestParam(required = false) String orderStatus,
                               @Parameter(description = "开始创建时间", required = false) @RequestParam(required = false) String startCreateTime,
                               @Parameter(description = "结束创建时间", required = false) @RequestParam(required = false) String endCreateTime,
                               @Parameter(description = "开始支付时间", required = false) @RequestParam(required = false) String startPayTime,
                               @Parameter(description = "结束支付时间", required = false) @RequestParam(required = false) String endPayTime,
                               @Parameter(description = "团购团长名称", required = false) @RequestParam(required = false) String promotionGroupUser,
                               @Parameter(description = "团购活动id", required = false) @RequestParam(required = false) String promotionGroupId,
                               @Parameter(description = "团购团长id", required = false) @RequestParam(required = false) String promotionGroupUserId,
                               @Parameter(description = "网点id", required = false) @RequestParam(required = false) String websitId,
                               @Parameter(description = "商品id", required = false) @RequestParam(required = false) String goodsId

    ) throws Exception {

        orderLogic.export2( request,  response,
                userId,  orderId,  productName,  userName,
                phone,  exchangeCode,  orderStatus,
                startCreateTime,  endCreateTime,
                startPayTime,  endPayTime,  promotionGroupUser,promotionGroupId,promotionGroupUserId,websitId,goodsId);
    }

}
