package com.gree.mall.contest.controller.pc.order;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.tax.RspOrderTax;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.order.OrderTaxLogic;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Slf4j
@RestController
@Tag(name = "发票管理", description = "发票管理")
@RequestMapping(value = "/pc/tax", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class OrderTaxController {

    private final OrderTaxLogic orderTaxLogic;

    @PostMapping("/update")
    @Operation(summary = "上传发票pdf地址")
    public ResponseHelper updateTax(
            @Parameter(description = "发票id", required = true) @RequestParam String orderTaxId,
            @Parameter(description = "发票pdf的url,使用urlencode编码", required = true) @RequestParam String taxlink
    ) throws RemoteServiceException {
        orderTaxLogic.updateTax(orderTaxId, taxlink);
        return ResponseHelper.success();
    }

    @PostMapping("/offline")
    @Operation(summary = "已线下处理")
    public ResponseHelper updateTax(
            @Parameter(description = "发票id", required = true) @RequestParam(required = true) String orderTaxId
    ) throws RemoteServiceException {
        orderTaxLogic.offline(orderTaxId);
        return ResponseHelper.success();
    }

    @GetMapping("/list")
    @Operation(summary = "发票列表")
    public ResponseHelper<Page<RspOrderTax>> pageTax(
            @Parameter(description = "开票状态true:已开发票,false:未开,不传:所有的", required = false) @RequestParam(required = false) Boolean status,
            @Parameter(description = "订单号", required = false) @RequestParam(required = false) String orderId,
            @Parameter(description = "页号", required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小", required = true) @RequestParam Integer pageSize,
            HttpServletRequest request
    ) throws Exception {

        IPage<RspOrderTax> page = orderTaxLogic.page(status, orderId, pageNum, pageSize, request);
        return ResponseHelper.success(page);
    }

    @PostMapping("/updateWkp")
    @Operation(summary = "修改未开票信息")
    public ResponseHelper updateTaxInfo(
            @Parameter(description = "orderTaxId", required = true) @RequestParam(required = true) String orderTaxId,
            @Parameter(description = "公司名称或个人名", required = true) @RequestParam(required = true) String name,
            @Parameter(description = "纳税人识别号", required = true) @RequestParam(required = true) String taxNo,
            @Parameter(description = "收票人名称", required = true) @RequestParam(required = true) String receiverName,
            @Parameter(description = "接收电话", required = true) @RequestParam(required = true) String receiverPhone,
            @Parameter(description = "接收邮箱", required = true) @RequestParam(required = true) String receiverEmail,
            @Parameter(description = "收票人地址", required = true) @RequestParam(required = true) String receiverAddress
    ) throws RemoteServiceException {
        orderTaxLogic.updateInfo(orderTaxId, name, taxNo, receiverName, receiverPhone, receiverEmail, receiverAddress);
        return ResponseHelper.success();
    }


    @GetMapping("/export")
    @Operation(summary = "导出发票信息")
    public void exportOrderTax(
            @Parameter(description = "开票状态true:已开发票,false:未开,不传:所有的", required = false) @RequestParam(required = false) Boolean status,
            @Parameter(description = "订单号", required = false) @RequestParam(required = false) String orderId,
            HttpServletResponse response,
            HttpServletRequest request
    ) throws Exception {
        orderTaxLogic.exportOrderTax(status, orderId, request, response);
    }


}
