package com.gree.mall.contest.logic.member;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.UserCoupon;
import com.gree.mall.contest.plus.service.UserCouponService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Service
@Slf4j
@RequiredArgsConstructor
public class UserCouponLogic {


    private final UserCouponService userCouponService;
    private final CommonLogic commonLogic;

    /**
     * 领取记录列表
     * @param couponName
     * @param couponType
     * @param pageNum
     * @param pageSize
     * @return
     */
    public IPage<UserCoupon> page(HttpServletRequest request, String couponName, String couponType, Integer pageNum, Integer pageSize) {
        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return userCouponService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(companyWechatIds), UserCoupon::getCompanyWechatId , companyWechatIds)
                .like(StringUtils.isNotBlank(couponName), UserCoupon::getCouponName, couponName)
                .eq(StringUtils.isNotBlank(couponName), UserCoupon::getCouponType, couponType)
                .page(new Page<>(pageNum, pageSize));
    }
}
