package com.gree.mall.contest.logic.order;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.ExcelData;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.order.OrderDetailBean;
import com.gree.mall.contest.bean.tax.OrderTaxExcelBean;
import com.gree.mall.contest.bean.tax.RspOrderTax;
import com.gree.mall.contest.commonmapper.CommonMapper;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.GoodsCategory;
import com.gree.mall.contest.plus.entity.OrderTax;
import com.gree.mall.contest.plus.service.OrderTaxService;
import com.gree.mall.contest.utils.RedisUtil;
import com.gree.mall.contest.utils.email.EmailUtilsNew;
import com.gree.mall.contest.utils.excel.ExcelUtils;
import com.gree.mall.contest.utils.http.HttpUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class OrderTaxLogic {
    private final OrderTaxService orderTaxService;
    private final RedisUtil redisUtil;
    private final CommonLogic commonLogic;
    private final CommonMapper commonMapper;
    private final OrderLogic orderLogic;
    @Value("${tax.token.appid}")
    private String clientId;
    @Value("${tax.token.appSecret}")
    private String clientSecret;
    @Value("${tax.token.url}")
    private String taxTokenUrl;
    @Value("${tax.invoice.title.url}")
    private String taxTitleUrl;


    private final static String EMAIL_SUBJECT = "发票";
    private final static String EMAIL_PREFIX = "请点击链接下载您的发票:\n\t";

    @Transactional
    public OrderTax saveTax(OrderTax orderTaxBean) {
        OrderDetailBean orderDetail = orderLogic.detail(orderTaxBean.getOrderId());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
        if (Integer.parseInt(sdf.format(orderDetail.getPayTime())) < 2022) {
            throw new RemoteServiceException("2022年之前的订单请联系客服开票");
        }
        String orderId = orderTaxBean.getOrderId();
        OrderDetailBean detail = orderLogic.detail(orderId);
        orderTaxBean.setCompanyWechatId(detail.getCompanyWechatId());
        orderTaxBean.setCompanyWechatName(detail.getCompanyWechatName());
        orderTaxBean.setWebsitId(detail.getWebsitId());
        orderTaxBean.setWebsitName(detail.getWebsitName());
        orderTaxBean.setOrderTaxId(null);
        orderTaxBean.setStatus(false);
        orderTaxBean.setCreateTime(new Date());
        if (orderTaxBean.getTaxNo() != null) {
            orderTaxBean.setTaxNo(orderTaxBean.getTaxNo().replaceAll(" ", ""));
        }
        orderTaxService.save(orderTaxBean);
        //标记订单已开发票
        orderLogic.tax(orderTaxBean.getOrderId(), orderTaxBean.getOrderTaxId());
        return orderTaxBean;
    }

    public void miniUpdateTax(OrderTax orderTaxBean) {
        orderTaxService.lambdaUpdate().eq(OrderTax::getOrderTaxId, orderTaxBean.getOrderTaxId())
                .set(StringUtils.isNotBlank(orderTaxBean.getAccount()), OrderTax::getAccount, orderTaxBean.getAccount())
                .set(orderTaxBean.getType() != null, OrderTax::getType, orderTaxBean.getType())
                .set(orderTaxBean.getTaxType() != null, OrderTax::getTaxType, orderTaxBean.getTaxType())
                .set(StringUtils.isNotBlank(orderTaxBean.getName()), OrderTax::getName, orderTaxBean.getName())
                .set(StringUtils.isNotBlank(orderTaxBean.getTaxNo()), OrderTax::getTaxNo, orderTaxBean.getTaxNo())
                .set(StringUtils.isNotBlank(orderTaxBean.getRegisterAddress()), OrderTax::getRegisterAddress, orderTaxBean.getRegisterAddress())
                .set(StringUtils.isNotBlank(orderTaxBean.getRegisterPhone()), OrderTax::getRegisterPhone, orderTaxBean.getRegisterPhone())
                .set(StringUtils.isNotBlank(orderTaxBean.getBank()), OrderTax::getBank, orderTaxBean.getBank())
                .set(StringUtils.isNotBlank(orderTaxBean.getAccount()), OrderTax::getAccount, orderTaxBean.getAccount())
                .set(StringUtils.isNotBlank(orderTaxBean.getReceiverPhone()), OrderTax::getReceiverPhone, orderTaxBean.getReceiverPhone())
                .set(StringUtils.isNotBlank(orderTaxBean.getReceiverEmail()), OrderTax::getReceiverEmail, orderTaxBean.getReceiverEmail())
                .set(StringUtils.isNotBlank(orderTaxBean.getContent()), OrderTax::getContent, orderTaxBean.getContent())
                .update();
    }

    public String delTax(String orderTaxId) {
        orderTaxService.lambdaUpdate()
                .eq(OrderTax::getOrderTaxId, orderTaxId)
                .remove();
        return orderTaxId;
    }

    public String sendTax(String orderTaxId) {
        OrderTax orderTax = orderTaxService.lambdaQuery()
                .eq(OrderTax::getOrderTaxId, orderTaxId)
                .one();

        String taxLink = orderTax == null ? null : orderTax.getTaxLink();
        if (StringUtils.isNotBlank(taxLink)) {
            new EmailUtilsNew().send(orderTax.getReceiverEmail(), EMAIL_SUBJECT, EMAIL_PREFIX + taxLink);
        }
        return taxLink;
    }


    public IPage<OrderTax> miniPage(String userId, Integer pageNum, Integer pageSize) {
        return orderTaxService.lambdaQuery()
                .eq(StringUtils.isNotBlank(userId), OrderTax::getUserId, userId)
                .orderByDesc(OrderTax::getCreateTime)
                .page(new Page(pageNum, pageSize));

    }

    public OrderTax detail(String orderTaxId) {
        return orderTaxService.lambdaQuery()
                .eq(StringUtils.isNotBlank(orderTaxId), OrderTax::getOrderTaxId, orderTaxId)
                .one();

    }

    public Object title(String companyName) throws RemoteServiceException {
        HashMap<String, String> params = new HashMap();
        HashMap<String, String> heads = new HashMap();
        heads.put("authorization_token", getTaxToken());
        params.put("companyName", companyName);
        String content = HttpUtils.requestGet(taxTitleUrl, params, heads);
        if (content == null) {
            throw new RemoteServiceException("网络错误");
        }
        Map<String, Object> resMap = JSONUtil.toBean(content, new TypeReference<HashMap<String, Object>>() {
        }, false);

        int code = (int) resMap.get("code");
        if (code == 1) {
            return resMap.get("data");
        }
        return null;
    }

    public String getTaxToken() throws RemoteServiceException {
        Object token = redisUtil.get(Constant.RedisPrefix.TOKEN_TAX);

        if (token != null) {
            return token.toString();
        }
        //过期就重新获取
        return this.deleteAndAddWxToken();
    }

    public String deleteAndAddWxToken() throws RemoteServiceException {
        HashMap<String, String> params = new HashMap();
        params.put("clientId", this.clientId);
        params.put("clientSecret", this.clientSecret);
        String content = HttpUtils.requestGet(taxTokenUrl, params, null);
        Map<String, Object> resMap = JSONUtil.toBean(content, new TypeReference<HashMap<String, Object>>() {
        }, false);

        int code = (int) resMap.get("code");
        if (code == 1) {
            String token = (String) resMap.get("data");
            redisUtil.set(Constant.RedisPrefix.TOKEN_TAX, token, 2 * 60 * 60);//2小时过期
            return token;
        }
        return null;
    }


    public void yonge(MultipartFile file) throws RemoteServiceException, IOException {

        List<Object> datas = ExcelUtils.importExcel(file);
        List<String> tableHeaders = (List<String>) datas.get(0);

        Map<String, List<List<Object>>> map = new HashMap<>();
        for (int i = 1; i < datas.size(); i++) {
            List<Object> row = (List<Object>) datas.get(i);
            //邮箱
            String email = (String) row.get(0);

            List<List<Object>> objects = map.get(email);
            if (objects == null) {
                objects = new ArrayList<>();
            }
            row.remove(0);
            objects.add(row);

            if (!map.containsKey(email)) {
                map.put(email, objects);
            }
        }
        for (String email : map.keySet()) {
            //String filePath = "B:/"+email+"-"+file.getName()+".xlsx";
            String filePath = "/app/tmp/" + email + "-" + file.getName() + ".xlsx";
            this.createExcel(tableHeaders, map.get(email), filePath);
            new EmailUtilsNew().send(email, file.getName(), "请查收", filePath);
        }

    }


    /**
     * 创建Excel
     */
    public void createExcel(List<String> tableHeaders, List<List<Object>> rows, String fileFullPath) {
        OutputStream out = null;
        try {
            out = new FileOutputStream(new File(fileFullPath));
            write(out, tableHeaders, rows);

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


    /**
     * 生成Excel文件的方法write
     *
     * @param outputStream
     */
    public void write(OutputStream outputStream, List<String> tableHeaders, List<List<Object>> rows) {
        // 初始一个workbook
        XSSFWorkbook workbook = new XSSFWorkbook();
        // 创建单个sheet
        XSSFSheet sheet = workbook.createSheet("sheet1");
        // 创建第一行，设置列名
        XSSFRow row0 = sheet.createRow(0);
        row0.setHeight((short) 400);             // 设置行高
        XSSFCell cell = null;                    // Excel的列
        XSSFFont font = workbook.createFont();// 设置字体
        font.setColor(HSSFFont.COLOR_NORMAL);           // 设置单元格字体的颜色
        XSSFCellStyle style = workbook.createCellStyle();// 样式
        // 设置边框颜色
        style.setTopBorderColor(IndexedColors.BLACK.getIndex());
        style.setBottomBorderColor(IndexedColors.BLACK.getIndex());
        style.setLeftBorderColor(IndexedColors.BLACK.getIndex());
        style.setRightBorderColor(IndexedColors.BLACK.getIndex());
        // 设置文本格式
        XSSFDataFormat format = workbook.createDataFormat();
        style.setDataFormat(format.getFormat("@"));

        short cellNumber = (short) tableHeaders.size();  // 表的列数
        for (int k = 0; k < cellNumber; k++) {
            if (k == 0) {
                continue;
            }
            int i = 0;
            i += k - 1;
            short b = 5000;
            cell = row0.createCell(i);          // 创建第0行第k列
            cell.setCellValue(tableHeaders.get(k));  // 设置第0行第k列的值
            sheet.setColumnWidth(i, b);         // 设置列的宽度
            style.setFont(font);                // 设置字体风格
            cell.setCellStyle(style);
        }


        XSSFCellStyle styleContext = workbook.createCellStyle();
        styleContext.setWrapText(true);
        // 设置边框颜色
        styleContext.setTopBorderColor(IndexedColors.BLACK.getIndex());
        styleContext.setBottomBorderColor(IndexedColors.BLACK.getIndex());
        styleContext.setLeftBorderColor(IndexedColors.BLACK.getIndex());
        styleContext.setRightBorderColor(IndexedColors.BLACK.getIndex());
        // 设置文本格式
        styleContext.setDataFormat(format.getFormat("@"));

        if (rows.size() > 0) {
            // 创建剩余行
            for (int rowIndex = 0; rowIndex < rows.size(); rowIndex++) {
                XSSFRow row = sheet.createRow(rowIndex + 1);
                List<Object> values = rows.get(rowIndex);
                // 创建多列
                for (int cellIndex = 0; cellIndex < values.size(); cellIndex++) {
                    cell = row.createCell(cellIndex);
                    cell.setCellStyle(styleContext);               // 设置风格
                    cell.setCellValue(values.get(cellIndex).toString());
                }
            }
        }
        try {
            workbook.write(outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void updateTax(String orderTaxId, String taxlink) {
        orderTaxService.lambdaUpdate().eq(StringUtils.isNotBlank(orderTaxId), OrderTax::getOrderTaxId, orderTaxId)
                .set(StringUtils.isNotBlank(taxlink), OrderTax::getTaxLink, taxlink)
                .set(OrderTax::getStatus, true)
                .update();
    }


    /**
     * 线下处理发票
     */
    public void offline(String orderTaxId) throws RemoteServiceException {
        OrderTax orderTax = orderTaxService.getById(orderTaxId);
        if (!orderTax.getTaxType()) {
            throw new RemoteServiceException("非专票不可处理");
        }
        orderTax.setStatus(true);
        orderTax.updateById();
    }


    public IPage<RspOrderTax> page(Boolean status, String orderId, Integer pageNum, Integer pageSize, HttpServletRequest request) {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        IPage page = orderTaxService.lambdaQuery()
                .like(StringUtils.isNotEmpty(orderId), OrderTax::getOrderId, orderId)
                .in(CollectionUtil.isNotEmpty(adminUser.getCompanyWechatIds()), OrderTax::getCompanyWechatId, adminUser.getCompanyWechatIds())
                ////.//in(CollectionUtils.isNotEmpty(adminUser.getAdminWebsitIds()),OrderTax::getWebsitId,adminUser.getAdminWebsitIds())
                .eq(status != null, OrderTax::getStatus, status)
                .orderByDesc(OrderTax::getCreateTime).page(new Page(pageNum, pageSize));

        List<RspOrderTax> list = new ArrayList<>();
        for (Object o : page.getRecords()) {
            OrderTax tax = (OrderTax) o;
            RspOrderTax rspOrderTax = new RspOrderTax();
            BeanUtils.copyProperties(tax, rspOrderTax);
            List<GoodsCategory> goodsCategories = commonMapper.queryGoodsCategoryByOrderId(tax.getOrderId());
            if (goodsCategories.size() == 0) {
                continue;
            }
            List<String> categoryNames = goodsCategories.stream().map(GoodsCategory::getName).distinct().collect(Collectors.toList());
            rspOrderTax.setCategoryNames(StringUtils.join(categoryNames, ","));
            list.add(rspOrderTax);
        }
        page.setRecords(list);
        return page;
    }

    public void updateInfo(String orderTaxId, String name, String taxNo, String receiverName, String receiverPhone, String receiveEmail, String receiverAddress) throws RemoteServiceException {

        //判断是否允许修改,已开票的不许修改
        OrderTax orderTax = orderTaxService.getById(orderTaxId);
        if (!orderTax.getStatus()) {
            throw new RemoteServiceException("已开票的不许修改");
        }
        //修改
        orderTaxService.lambdaUpdate()
                .eq(OrderTax::getStatus, false)
                .eq(OrderTax::getOrderTaxId, orderTaxId)
                .set(StringUtils.isNotEmpty(name), OrderTax::getName, name)
                .set(StringUtils.isNotEmpty(taxNo), OrderTax::getTaxNo, taxNo)
                .set(StringUtils.isNotEmpty(receiverName), OrderTax::getReceiverName, receiverName)
                .set(StringUtils.isNotEmpty(receiverPhone), OrderTax::getReceiverPhone, receiverPhone)
                .set(StringUtils.isNotEmpty(receiveEmail), OrderTax::getReceiverEmail, receiveEmail)
                .set(StringUtils.isNotEmpty(receiverAddress), OrderTax::getReceiverAddress, receiverAddress)
                .update();
    }

    public void exportOrderTax(Boolean status, String orderId, HttpServletRequest request, HttpServletResponse response) throws Exception {

        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();
        List<String> adminWebsitIds = adminUser.getAdminWebsitIds();

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat();
        List<List<Object>> rowList = new ArrayList<>();

        ExcelData excelData = new ExcelData();
        excelData.setRows(rowList);
        excelData.setName("发票");
        excelData.setTitles(
                Arrays.asList(
                        "发票编号", "订单编号", "用户编号", "公司/个人", "专票/普票", "公司名称或个人名", "纳税人识别号",
                        "公司注册地址", "公司注册电话", "开户银行", "银行账号", "接收电话", "接收邮箱", "收票人名称",
                        "收票人地址", "发票内容", "已开票/未开票", "电子发票下载链接", "创建时间", "商品分类", "订单支付时间"
                )
        );
        adminWebsitIds = null;
        List<OrderTaxExcelBean> orderTaxExcelBeanList = commonMapper.queryOrderTaxExcelMsg(companyWechatIds, adminWebsitIds, status, orderId);
        for (OrderTaxExcelBean orderTaxExcelBean : orderTaxExcelBeanList
        ) {
            List<Object> row = new ArrayList<>();
            rowList.add(row);

            row.add(orderTaxExcelBean.getOrderTaxId());
            row.add(orderTaxExcelBean.getOrderId());
            row.add(orderTaxExcelBean.getUserId());
            row.add(orderTaxExcelBean.getType() == null ? "" : (orderTaxExcelBean.getType() ? "公司" : "个人"));
            row.add(orderTaxExcelBean.getTaxType() == null ? "" : (orderTaxExcelBean.getTaxType() ? "专票" : "普票"));
            row.add(orderTaxExcelBean.getName());
            row.add(orderTaxExcelBean.getTaxNo());
            row.add(orderTaxExcelBean.getRegisterAddress());
            row.add(orderTaxExcelBean.getRegisterPhone());
            row.add(orderTaxExcelBean.getBank());
            row.add(orderTaxExcelBean.getAccount());
            row.add(orderTaxExcelBean.getReceiverPhone());
            row.add(orderTaxExcelBean.getReceiverEmail());
            row.add(orderTaxExcelBean.getReceiverName());
            row.add(orderTaxExcelBean.getReceiverAddress());
            row.add(orderTaxExcelBean.getContent());
            row.add(orderTaxExcelBean.getStatus() == null ? "" : (orderTaxExcelBean.getStatus() ? "已开" : "待开"));
            row.add(orderTaxExcelBean.getTaxLink());
            row.add(orderTaxExcelBean.getCreateTime() == null ? "" : DateUtil.formatDate(orderTaxExcelBean.getCreateTime()));
            row.add(orderTaxExcelBean.getCategoryNames());
            row.add(orderTaxExcelBean.getPayTime() == null ? "" : DateUtil.formatDate(orderTaxExcelBean.getPayTime()));
        }


        ExcelUtils.exportExcel(request, response, String.format("发票导出%s.xlsx", simpleDateFormat.format(new Date())), excelData);

    }
}
